@@ -1,31 +1,42 @@
-# Copyright (C) 2005-2013 Quentin Sculo <squentin@free.fr>
+# Copyright (C) 2005-2015 Quentin Sculo <squentin@free.fr>
 #
 # This file is part of Gmusicbrowser.
 # Gmusicbrowser is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3, as
 # published by the Free Software Foundation
 
+BEGIN
+{	require GStreamer;
+	$::gstreamer_version='0.10';
+	die "Needs GStreamer version >= 0.05\n" if GStreamer->VERSION<.05;
+	die "Can't initialize GStreamer.\n" unless GStreamer->init_check;
+	GStreamer->init;
+	my $reg=GStreamer::Registry->get_default;
+	$Play_GST::reg_keep=$reg if GStreamer->CHECK_VERSION(0,10,4); #work-around to keep the register from being finalized in gstreamer<0.10.4 (see http://bugzilla.gnome.org/show_bug.cgi?id=324818)
+	$reg->lookup_feature('playbin') or die "gstreamer plugin 'playbin' not found.\nYou need to install at least gst-plugins-base.\n";
+}
+
 package Play_GST;
 use strict;
 use warnings;
 
-my ($GST_ok,$GST_visuals_ok,$GST_EQ_ok,$GST_RG_ok,$playbin2_ok); our $GST_RGA_ok;
+my ($GST_visuals_ok,$GST_EQ_ok,$GST_RG_ok,$playbin2_ok); our $GST_RGA_ok;
 my ($PlayBin,$Sink);
-my ($WatchTag,$Skip);
+my ($WatchTag,$Skip,$StateAfterSkip);
 my (%Plugins,%Sinks);
 my ($VSink,$visual_window);
 my $AlreadyNextSong;
 my $RG_dialog;
+my ($VolumeBusy,$VolumeHasChanged);
 
 $::PlayPacks{Play_GST}=1; #register the package
 
-my $reg_keep; #work-around to keep the register from being finalized in gstreamer<0.10.4 (see http://bugzilla.gnome.org/show_bug.cgi?id=324818)
-
 
 BEGIN
 { %Sinks=
   (	autoaudio	=> { name => _"auto detect", },
 	oss		=> { option => 'device' },
+	oss4		=> { option => 'device' },
 	esd		=> { option => 'host'},
 	alsa		=> { option => 'device'},
 	artsd		=> {},
@@ -37,64 +48,49 @@ BEGIN
 	osxaudio	=> {},
 	directsound	=> {},
 	#alsaspdif	=> { name => "alsa S/PDIF", option => 'card' },
-	#oss4		=> {},
 	#nas		=> {},
   );
-  %Plugins=(	mp3	=> 'mad',	oga => 'vorbisdec',	ape => 'ffdec_ape',
-		flac	=> 'flacdec',	mpc => 'musepackdec',	wv => 'wavpackdec',
-		m4a	=> 'faad',
-	);
-  my $error;
-  my $reg;
-  if (grep -f $_.'/GStreamer.pm',@INC)
-  {	eval {require GStreamer};
-	$error="Needs GStreamer version >= 0.05\n" if !$@ && GStreamer->VERSION<.05;
-	if (!$@ && !$error && GStreamer->init_check)
-	{	GStreamer->init;
-		$reg=GStreamer::Registry->get_default;
-		$reg_keep=$reg if GStreamer->CHECK_VERSION(0,10,4);
-		$playbin2_ok= $reg->lookup_feature('playbin2');
-		if ( $playbin2_ok || $reg->lookup_feature('playbin') ) { $GST_ok=1; }
-		else { $error="gstreamer plugin 'playbin' not found\nYou need to install at least gst-plugins-base\n"; }
-	}
-	else { $error=$@? $@ : "Can't initialize GStreamer.\n"; }
-  }
-  else {$error="GStreamer.pm not found\n";}
-  if ($error) {warn "$error -> gstreamer output won't be available.\n"}
-  if ($GST_ok)
-  {	$::Options{gst_sink}||= (grep ($reg->lookup_feature($_.'sink'), qw/autoaudio gconfaudio alsa esd pulse oss/),'alsa')[0]; #find a default sink
-	if ($reg->lookup_feature('equalizer-10bands')) { $GST_EQ_ok=1; }
-	else {warn "gstreamer plugin 'equalizer-10bands' not found -> equalizer not available\n";}
-	if ($reg->lookup_feature('rglimiter') && $reg->lookup_feature('rgvolume')) { $GST_RG_ok=1; }
-	else {warn "gstreamer plugins 'rglimiter' and/or 'rgvolume' not found -> replaygain not available\n";}
-	if ($reg->lookup_feature('rganalysis')) { $GST_RGA_ok=1; }
-	else {warn "gstreamer plugins 'rganalysis' not found -> replaygain analysis not available\n";}
-	$GST_visuals_ok=1;
-	eval {require GStreamer::Interfaces};
-	if ($@) {warn "GStreamer::Interfaces perl module not found -> visuals not available\n"; $GST_visuals_ok=0;}
-	unless ($reg->lookup_feature('ximagesink'))
-	{	warn "gstreamer plugin 'ximagesink' not found -> visuals not available\n"; $GST_visuals_ok=0;
-	}
-  }
+  %Plugins=(	mp3 => 'flump3dec mad mpg123audiodec avdec_mp3',
+		oga => 'vorbisdec',			flac=> 'flacdec',
+		ape => 'avdec_ape ffdec_ape',		wv  => 'wavpackdec',
+		mpc => 'musepackdec avdec_mpc8',	m4a => 'faad',
+  );
 
+  my $reg=GStreamer::Registry->get_default;
+  $playbin2_ok= $reg->lookup_feature('playbin2');
+  if ($reg->lookup_feature('equalizer-10bands')) { $GST_EQ_ok=1; }
+  else {warn "gstreamer plugin 'equalizer-10bands' not found -> equalizer not available\n";}
+  if ($reg->lookup_feature('rglimiter') && $reg->lookup_feature('rgvolume')) { $GST_RG_ok=1; }
+  else {warn "gstreamer plugins 'rglimiter' and/or 'rgvolume' not found -> replaygain not available\n";}
+  if ($reg->lookup_feature('rganalysis')) { $GST_RGA_ok=1; }
+  else {warn "gstreamer plugins 'rganalysis' not found -> replaygain analysis not available\n";}
+  $GST_visuals_ok=1;
+  eval {require GStreamer::Interfaces};
+  if ($@) {warn "GStreamer::Interfaces perl module not found -> visuals not available\n"; $GST_visuals_ok=0;}
+  unless ($reg->lookup_feature('ximagesink'))
+  {	warn "gstreamer plugin 'ximagesink' not found -> visuals not available\n"; $GST_visuals_ok=0;
+  }
 }
 
 sub supported_formats
-{	return '' unless $GST_ok;
-	my $reg=GStreamer::Registry->get_default;
-	return grep $reg->lookup_feature($Plugins{$_}), keys %Plugins;
+{	my $reg=GStreamer::Registry->get_default;
+	my @found;
+	for my $type (keys %Plugins)
+	{	push @found, $type if grep $reg->lookup_feature($_), split / +/, $Plugins{$type};
+	}
+	return @found;
 }
 sub supported_sinks
-{	return {} unless $GST_ok;
-	my $reg=GStreamer::Registry->get_default;
+{	my $reg=GStreamer::Registry->get_default;
 	$Sinks{$_}{ok}= ! !$reg->lookup_feature($_.'sink') for keys %Sinks;
 	#$::Options{gst_sink}='autoaudio' unless $Sinks{$::Options{gst_sink}};
 	return {map { $_ => $Sinks{$_}{name}||$_ } grep $Sinks{$_}{ok}, keys %Sinks};
 }
 
 sub init
-{	return undef unless $GST_ok;
-	#createPlayBin();
+{	my $reg=GStreamer::Registry->get_default;
+	$::Options{gst_sink}='' unless $reg->lookup_feature( ($::Options{gst_sink}||'').'sink' );
+	$::Options{gst_sink}||= (grep ($reg->lookup_feature($_.'sink'), qw/autoaudio gconfaudio pulse alsa esd oss oss4/),'autoaudio')[0]; #find a default sink
 	return bless { EQ=>$GST_EQ_ok, visuals => $GST_visuals_ok, RG=>$GST_RG_ok },__PACKAGE__;
 }
 
@@ -103,9 +99,10 @@ sub createPlayBin
 	my $pb= $playbin2_ok ? 'playbin2' : 'playbin';
 	$PlayBin=GStreamer::ElementFactory->make($pb => 'playbin'); #FIXME only the first one used works
 	$PlayBin->set('flags' => [qw/audio soft-volume/]) if $playbin2_ok;
-	$PlayBin->set(volume => ($::Volume/100)**3); 	#initialize volume, use a cubic volume scale
+	SetVolume(undef,''); #initialize volume
 	my $bus=$PlayBin->get_bus;
 	$bus->add_signal_watch;
+	$PlayBin->signal_connect("notify::volume" => sub { Glib::Idle->add(\&VolumeChanged) unless $VolumeHasChanged++; },100000) if $Glib::VERSION >= 1.251 && $::Options{gst_monitor_pa_volume}; #not stable with older version of perl-glib due to bug #620099 (https://bugzilla.gnome.org/show_bug.cgi?id=620099), and still not quite stable
 #	$bus->signal_connect('message' => \&bus_message);
 	$bus->signal_connect('message::eos' => \&bus_message_end);
 	$bus->signal_connect('message::error' => \&bus_message_end,1);
@@ -136,8 +133,22 @@ sub bus_message_end
 	else		{ ::end_of_file(); }
 }
 
-sub bus_message_state_changed
-{	SkipTo(undef,$Skip) if $Skip;
+# when a song starts playing, notify::volume callbacks are called, which causes glib to hang (see https://bugzilla.gnome.org/show_bug.cgi?id=620099#c11) if a skip is done at the same moment
+# using freeze_notify until the skip is done mostly avoid the problem
+# setting state to paused until the skip is done also mostly avoid the problem
+# but the hang still happens in some cases, in particular when using the scroll wheel to change the position in the song, and also very rarely when starting a song
+my $StateChanged;
+sub bus_message_state_changed	# used to wait for the right state to do the skip
+{	return unless $Skip;
+	return if $StateChanged;
+	$StateChanged=1;
+	$PlayBin->freeze_notify unless $PlayBin->{notify_frozen}; $PlayBin->{notify_frozen}=1; #freeze notify until skip is done
+	Glib::Idle->add(sub
+	{	SkipTo(undef,$Skip) if $Skip; # this will only skip if the state is right, else will wait for another state change
+		unless ($Skip) { $PlayBin->thaw_notify if delete $PlayBin->{notify_frozen}; } #if skip is done, unfreeze
+		$StateChanged=0;
+		0;
+	});
 }
 
 sub Close
@@ -156,10 +167,28 @@ sub SetVolume
 	elsif	($set=~m/(\d+)/)	{ $::Volume =$1; }
 	$::Volume=0   if $::Volume<0;
 	$::Volume=100 if $::Volume>100;
-	$PlayBin->set(volume => ($::Volume/100)**3) if $PlayBin; 	#use a cubic volume scale
-	::HasChanged('Vol');
+	$VolumeBusy=1;
+	$PlayBin->set(volume => ( ($::Mute||$::Volume) /100)**3, mute => !!$::Mute) if $PlayBin; 	#use a cubic volume scale
+	$VolumeBusy=0;
 	$::Options{Volume}=$::Volume;
 	$::Options{Volume_mute}=$::Mute;
+	::QHasChanged('Vol');
+}
+sub VolumeChanged
+{	$VolumeHasChanged=0;
+	return 0 if $VolumeBusy;
+	return 0 unless $PlayBin;
+	my ($volume,$mute)= $PlayBin->get('volume','mute');
+	$volume= $volume ** (1/3) *100; #use a cubic volume scale
+	$volume= sprintf '%d',$volume;
+	$volume=100 if $volume>100;
+	#return 0 unless $volume!=$::Volume || ($mute xor !!$::Mute);
+	if ($mute)	{ $::Mute=$volume; $::Volume=0; }
+	else		{ $::Mute=0; $::Volume=$volume; }
+	$::Options{Volume}=$::Volume;
+	$::Options{Volume_mute}=$::Mute;
+	::QHasChanged('Vol');
+	0; #called from an idle
 }
 
 sub SkipTo
@@ -168,14 +197,17 @@ sub SkipTo
 	my ($result,$state,$pending)=$PlayBin->get_state(0);
 	return if $result eq 'async'; #when song hasn't started yet, needs to wait until it has started before skipping
 	$PlayBin->seek(1,'time','flush','set', $Skip*1_000_000_000,'none',0);
+	if ($StateAfterSkip) { $PlayBin->set_state($StateAfterSkip); $StateAfterSkip=undef; }
 	$Skip=undef;
 }
 
 sub Pause
 {	$PlayBin->set_state('paused');
+	$StateAfterSkip=undef;
 }
 sub Resume
 {	$PlayBin->set_state('playing');
+	$StateAfterSkip=undef;
 }
 
 sub check_sink
@@ -279,9 +311,11 @@ sub Play
 	{	$visual_window->realize unless $visual_window->window;
 		if (my $w=$visual_window->window) { $VSink->set_xwindow_id($w->XID); }
 	}
-	warn "playing $file\n";
+	warn "playing $file\n" if $::Verbose;
 	set_file($file);
-	$PlayBin -> set_state('playing');
+	my $newstate='playing'; $StateAfterSkip=undef;
+	if ($Skip) { $newstate='paused'; $StateAfterSkip='playing'; }
+	$PlayBin->set_state($newstate);
 	$WatchTag=Glib::Timeout->add(500,\&_UpdateTime) unless $WatchTag;
 }
 sub set_file
@@ -298,6 +332,7 @@ sub set_equalizer_preamp
 	my $preamp=$PlayBin->get_by_name('equalizer-preamp');
 	$preamp->set( volume => $volume**3) if $preamp;
 	$::Options{gst_equalizer_preamp}=$volume;
+	::HasChanged('Equalizer','preamp');
 }
 sub set_equalizer
 {	my (undef,$band,$val)=@_;
@@ -308,6 +343,7 @@ sub set_equalizer
 	::setlocale(::LC_NUMERIC, 'C');
 	$::Options{gst_equalizer}=join ':',@vals;
 	::setlocale(::LC_NUMERIC, '');
+	::HasChanged('Equalizer','value');
 }
 sub EQ_Get_Range
 {	createPlayBin() unless $PlayBin;
@@ -441,6 +477,7 @@ sub Stop
 	#warn "stop: state: $result,$state,$pending\n";
 	#return if $state eq 'null' and $pending eq 'void-pending';
 	$PlayBin->set_state('null') if $PlayBin;
+	$StateAfterSkip=undef;
 	#warn "--stop\n";
 }
 
@@ -464,6 +501,11 @@ sub AdvancedOptions
 	my $gapless= ::NewPrefCheckButton(gst_gapless => _"enable gapless (experimental)", cb=> $modif_cb);
 	$gapless->set_sensitive(0) unless $playbin2_ok;
 	$vbox->pack_start($gapless,::FALSE,::FALSE,2);
+
+	my $monitor_volume= ::NewPrefCheckButton(gst_monitor_pa_volume => _("Monitor the pulseaudio volume").' '._("(unstable)"), cb=> $modif_cb, tip=>_"Makes gmusicbrowser monitor its pulseaudio volume, so that external changes to its volume are known.");
+	$monitor_volume->set_sensitive(0) unless $Glib::VERSION >= 1.251;
+	$vbox->pack_start($monitor_volume,::FALSE,::FALSE,2);
+
 	my $sg1=Gtk2::SizeGroup->new('horizontal');
 	my $custom= ::NewPrefEntry(gst_custom => _"Custom pipeline", cb=>$modif_cb, sizeg1 => $sg1, expand => 1, tip => _"Insert this pipeline before the audio sink", history => 'gst_custom_history');
 	$vbox->pack_start($custom,::FALSE,::FALSE,2);
@@ -487,7 +529,7 @@ my $RGA_songmenu=
  [	{ label => _"Scan this file",			code => sub { Analyse ($_[0]{IDs}); },		onlyone => 'IDs', },
 	{ label => _"Scan per-file track gain",		code => sub { Analyse ($_[0]{IDs}); },		onlymany=> 'IDs', },
 	{ label => _"Scan using tag-defined album", 	code => sub { Analyse_byAlbum ($_[0]{IDs}); },	onlymany=> 'IDs', },
-	{ label => _"Scan as an album",			code => sub { Analyse(join " ",@{ $_[0]{IDs} }); },	onlymany=> 'IDs', },
+	{ label => _"Scan as an album",			code => sub { Analyse([join ' ',@{ $_[0]{IDs} }]); },	onlymany=> 'IDs', },
  ],
 };
 push @::SongCMenu,$RGA_songmenu;
@@ -518,7 +560,7 @@ sub Analyse_byAlbum
 	for my $aid (keys %$hash)
 	{	my $IDs= $hash->{$aid};
 		if (@$IDs<2 || Songs::Gid_to_Get('album',$aid) eq '') { push @list, @$IDs; } #no album name or only 1 song in album => push as single songs
-		else { push @list, join ' ',$IDs; } # push as an album
+		else { push @list, join ' ',@$IDs; } # push as an album
 	}
 	Analyse(\@list);
 }
@@ -592,7 +634,7 @@ sub process_next
 		my $f= Songs::Get($ID,'fullfilename_raw');
 		::_utf8_on($f); # pretend it's utf8 to prevent a conversion to utf8 by the bindings
 		$RGA_pipeline->{ID}=$ID;
-		warn "analysing [$ID] $f\n";
+		warn "Analysing [$ID] $f\n" if $::Verbose;
 		$RGA_pipeline->get_by_name('filesrc')->set(location => $f);
 		$rganalysis->set_locked_state(0);
 		$RGA_pipeline->set_state('playing');
@@ -657,7 +699,7 @@ sub WriteRGtags
 	my $modif=shift @towrite;
 	Songs::Set($ID, $modif,
 		abortmsg	=> _"Abort ReplayGain analysis",
-		error_prefix	=> _"Error writing replaygain tags :\n",
+		errormsg	=> _"Error while writing replaygain data",
 		abortcb		=> \&StopAnalysis,
 		callback_finish	=> \&WriteRGtags,
 	);
@@ -674,14 +716,12 @@ use Socket;
 use constant { EOL => "\015\012" };
 our @ISA=('Play_GST');
 
-our %sockets;
-my ($stream,$Server);
+my (%sockets,$stream,$Server);
 
 $::PlayPacks{Play_GST_server}=1; #register the package
 
 sub init
-{	return undef unless $GST_ok;
-	my $ok=1;
+{	my $ok=1;
 	my $reg=GStreamer::Registry->get_default;
 	for my $feature (qw/multifdsink lame audioresample audioconvert/)
 	{	next if $reg->lookup_feature($feature);
@@ -805,5 +845,8 @@ sub Connection
 	return 1;	#keep listening
 }
 
+sub get_connections
+{	return map $sockets{$_}[2], grep $sockets{$_}[1],keys %sockets;
+}
 
 1;
