@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Quentin Sculo <squentin@free.fr>
+# Copyright (C) 2005-2015 Quentin Sculo <squentin@free.fr>
 #
 # This file is part of Gmusicbrowser.
 # Gmusicbrowser is free software; you can redistribute it and/or modify
@@ -43,7 +43,7 @@ sub Read
 	return unless $format;
 	my ($package,$formatstring,$plist)=@$format;
 	my $filetag= eval { $package->new($file,$findlength); }; #filelength==1 -> may return estimated length (mp3 only)
-	unless ($filetag) { warn $@ if $@; warn "can't read tags for $file\n"; return }
+	unless ($filetag) { warn $@ if $@; warn "Can't read tags for $file\n"; return }
 
 	::setlocale(::LC_NUMERIC, 'C');
 	my @taglist;
@@ -245,7 +245,8 @@ sub FMPS_hash_write
 }
 
 sub PixFromMusicFile
-{	my ($file,$nb,$quiet)=@_;
+{	my ($file,$nb,$quiet,$return_number)=@_;
+	if ($file=~s/:(\w+)$//) {$nb=$1} # index can be specified as argument or in the filename
 	my ($h)=Read($file,0,'embedded_pictures');
 	return unless $h;
 	my $pix= $h->{embedded_pictures};
@@ -269,6 +270,7 @@ sub PixFromMusicFile
 	}
 	elsif ($nb>$#$pix) { $nb=0 }
 
+	return $nb if $return_number;
 	return ref $pix->[0] ? $pix->[$nb][3] : $pix->[$nb];
 }
 
@@ -284,14 +286,24 @@ sub GetLyrics
 
 sub WriteLyrics
 {	my ($ID,$lyrics)=@_;
-	Write($ID, [embedded_lyrics=>$lyrics], \&::Retry_Dialog);
+	Write($ID, [embedded_lyrics=>$lyrics], sub
+	 {	my ($syserr,$details)= Error_Message(@_);
+		return ::Retry_Dialog($syserr, _"Error writing lyrics", details=>$details, ID=>$ID);
+	 });
+}
+
+#convert error details from tag writing to translated string with utf8 filenames
+sub Error_Message
+{	my ($syserr,$type,$file)=@_;
+	my $details= $type eq 'openwrite' ?
+		::__x(_"Error opening '{file}' for writing.",file=>::filename_to_utf8displayname($file)) :
+		'Unknown error'; #currently $type is always "openwrite"
+	return $syserr,$details;
 }
 
 package MassTag;
-use Gtk2;
-use constant
-{	TRUE  => 1, FALSE => 0,
-};
+
+use constant { TRUE  => 1, FALSE => 0, };
 
 our @FORMATS;
 our @FORMATS_user;
@@ -351,7 +363,7 @@ INIT
 # @FORMATS=@tmp;
 }
 
-use base 'Gtk2::VBox';
+use base 'Gtk2::Box';
 sub new
 {	my ($class,@IDs) = @_;
 	@IDs= ::uniq(@IDs);
@@ -373,7 +385,7 @@ sub new
 			$folder=_"different folders";
 			$folder.= "\n". ::__x(_"(common parent folder : {common})",common=> $displaysub->($common) ) if length($common)>5;
 		}
-		my $text= ::__("%d file in {folder}","%d files in {folder}",scalar@IDs);
+		my $text= ::__n("%d file in {folder}","%d files in {folder}",scalar@IDs);
 		$text= ::__x($text, folder => ::MarkupFormat('<small>%s</small>', $displaysub->($folder) ) );
 		my $labelfile = Gtk2::Label->new;
 		$labelfile->set_markup($text);
@@ -396,10 +408,19 @@ sub new
 		$table->attach($widget,$col++,$col,$row,$row+1,['fill','expand'],'shrink',3,1);
 	}
 
-	$self->pack_start($table, FALSE, TRUE, 2);
+	my $vpaned= $self->{vpaned}=Gtk2::VPaned->new;
+	$self->add($vpaned);
+	my $sw=Gtk2::ScrolledWindow->new;
+	$sw->set_shadow_type('none');
+	$sw->set_policy('never', 'automatic');
+	$sw->add_with_viewport($table);
+	$sw->show_all;
+	$sw->set_size_request(-1,$table->size_request->height);
+	$vpaned->pack1($sw,FALSE,TRUE);
 
 	# do not add per-file part if LOTS of songs, building the GUI would be too long anyway
 	$self->add_per_file_part unless @IDs>1000;
+	$self->set_size_request(-1,400); #to allow resizing the window to a small height in spite of the height request of $sw
 	return $self;
 }
 
@@ -429,10 +450,8 @@ sub add_per_file_part
 		#my $item=Gtk2::CheckMenuItem->new(_"Select files");
 		#$item->signal_connect(activate => sub { $self->add_selectfile_column });
 		#$menu->append($item);
-		$menu->show_all;
 		$BSelFields->signal_connect( button_press_event => sub
-			{	my $event=$_[1];
-				$menu->popup(undef,undef,\&::menupos,undef,$event->button, $event->time);
+			{	::PopupMenu($menu,event=>$_[1]);
 			});
 		#$self->pack_start($menubar, FALSE, FALSE, 2);
 		#$perfile_table->attach($menubar,7,8,0,1,'fill','shrink',1,1);
@@ -456,10 +475,8 @@ sub add_per_file_part
 			$item->signal_connect(activate => $menu_cb,$ref->{for_all});
 			$menu->append($item) if $ref->{for_all};
 		}
-		$menu->show_all;
 		$Btools->signal_connect( button_press_event => sub
-			{	my $event=$_[1];
-				$menu->popup(undef,undef,\&::menupos,undef,$event->button, $event->time);
+			{	::PopupMenu($menu,event=>$_[1]);
 			});
 	}
 
@@ -468,10 +485,21 @@ sub add_per_file_part
 		undef,_"Clear selected fields");
 
 	my $sw = Gtk2::ScrolledWindow->new;
-	$sw->set_shadow_type('etched-in');
+	$sw->set_shadow_type('none');
 	$sw->set_policy('automatic', 'automatic');
 	$sw->add_with_viewport($perfile_table);
-	$self->pack_start($sw, TRUE, TRUE, 4);
+
+	# expander to hide/show the per-file part
+	my $exp_label=Gtk2::Label->new_with_format("<b>%s</b>",_"Per-song values");
+	my $expander=Gtk2::Expander->new;
+	$expander->set_expanded(TRUE);
+	$expander->set_label_widget($exp_label);
+	$expander->signal_connect(activate=>sub { my $on= !$_[0]->get_expanded; $_->set_visible($on) for $sw,$BSelFields; });
+
+	$self->{vpaned}->pack2( ::Vpack('compact',[$expander,$BSelFields],'_',$sw), TRUE,FALSE);
+	my $vsizegroup=Gtk2::SizeGroup->new('vertical');
+	$vsizegroup->add_widget($_) for $exp_label,$BSelFields; # so that they are aligned
+	$sw->set_size_request(-1,$exp_label->size_request->height); # so that the expander is always visible
 
 	my $store= Gtk2::ListStore->new('Glib::String','Glib::Scalar');
 	$self->{autofill_combo}= my $Bautofill=Gtk2::ComboBox->new($store);
@@ -485,7 +513,7 @@ sub add_per_file_part
 	my $checkOBlank=Gtk2::CheckButton->new(_"Auto fill only blank fields");
 	$self->{AFOBlank}=$checkOBlank;
 	my $hbox=Gtk2::HBox->new;
-	$hbox->pack_start($_, FALSE, FALSE, 0) for $BSelFields,Gtk2::VSeparator->new,$Bautofill,$BClear,$checkOBlank,$Btools,
+	$hbox->pack_start($_, FALSE, FALSE, 0) for Gtk2::VSeparator->new,$Bautofill,$BClear,$checkOBlank,$Btools,
 	$self->pack_start($hbox, FALSE, FALSE, 4);
 }
 
@@ -520,18 +548,20 @@ sub add_column
 		   $_->set_sensitive($active) for @entries;
 		});
 
-	# add auto-increment button to track column
-	if ($field eq 'track' && 1)	#good idea ? make entries a bit large :(
+	# add auto-increment/auto-complete button to track/disc/year columns
+	if ($field eq 'track' || $field eq 'disc' || $field eq 'year')
 	{	#$_->set_alignment(1) for @entries;
-		my $increment=sub
-		 {	my $i=1;
+		my ($increment,$tip)= $field eq 'track' ? (1,_"Auto-increment track numbers") : (0,_"Copy missing values from previous line");
+		my $autosub=sub
+		 {	my $i= $field ne 'year' ? 1 : 0;
 			for my $e (@entries)
 			{	my $here=$e->get_text;
-				if ($here && $here=~m/^\d+$/) { $i=$here; } else { $e->set_text($i) }
-				$i++;
+				if	($here && $here=~m/^\d+$/) { $i=$here; }
+				elsif	($i>0)	{ $e->set_text($i) }
+				$i++ if $increment;
 			}
 		 };
-		my $button=::NewIconButton('gtk-go-down',undef,$increment,'none',_"Auto-increment track numbers");
+		my $button=::NewIconButton('gtk-go-down',undef,$autosub,'none',$tip);
 		$button->set_border_width(0);
 		$button->set_size_request();
 		$check->signal_connect( toggled => sub { $button->set_sensitive($_[0]->get_active) });
@@ -571,7 +601,7 @@ sub add_selectfile_column
 sub scroll_to_entry
 {	my $ent=$_[0];
 	if (my $sw=::find_ancestor($ent,'Gtk2::Viewport'))
-	{	my ($x,$y,$w,$h)=$ent->window->get_geometry;
+	{	my ($x,$y,$w,$h)= $ent->allocation->values;
 		$sw->get_hadjustment->clamp_page($x,$x+$w);
 		$sw->get_vadjustment->clamp_page($y,$y+$h);
 	};
@@ -748,9 +778,14 @@ sub new
 	$sg->add_widget($_) for $label_format,$label_re;
 	my $bbox= Gtk2::HButtonBox->new;
 	$bbox->add($_) for $button_del, $button_add, $button_new;
+	my $sw= ::new_scrolledwindow($treeview,'etched-in');
+	 $sw->set_size_request(150,-1); #give the list a minimum width
 	my $table= ::MakeReplaceTable('taAlCyndgL', A=>Songs::FieldName('album_artist_raw'));	#AutoFillFields
-	my $hbox= ::Hpack($treeview,'_',[[$label_format,'_',$entry_format],$table,$check_re,[$label_re,'_',$entry_re],$error,$preview,'-',$bbox]);
-	$self->vbox->add($hbox);
+	my $hbox= ::Vpack([$label_format,'_',$entry_format],$table,$check_re,[$label_re,'_',$entry_re],$error,$preview,'-',$bbox);
+	my $hpaned= Gtk2::HPaned->new;
+	 $hpaned->pack1($sw,1,1);
+	 $hpaned->pack2($hbox,1,0);
+	$self->vbox->add($hpaned);
 
 	::set_drag($preview, dest => [::DRAG_ID,\&song_dropped]);
 	$entry_format->signal_connect(changed=> \&entry_changed);
@@ -906,7 +941,6 @@ sub cursor_changed_cb
 
 
 package GMB::TagEdit::EntryString;
-use Gtk2;
 use base 'Gtk2::Entry';
 
 sub new
@@ -927,18 +961,14 @@ sub tool
 }
 
 package GMB::TagEdit::EntryText;
-use Gtk2;
-use base 'Gtk2::VBox';
+use base 'Gtk2::Box';
 
 sub new
 {	my ($class,$field,$IDs) = @_;
 	my $self = bless Gtk2::VBox->new, $class;
-	my $sw = Gtk2::ScrolledWindow->new;
-	 $sw->set_shadow_type('etched-in');
-	 $sw->set_policy('automatic','automatic');
 	my $textview= $self->{textview}= Gtk2::TextView->new;
 	$textview->set_size_request(100,($textview->create_pango_layout("X")->get_pixel_size)[1]*4); #request 4 lines of height
-	$sw->add($textview);
+	my $sw= ::new_scrolledwindow($textview,'etched-in');
 	$self->add($sw);
 	my $val;
 	if (ref $IDs)
@@ -981,7 +1011,6 @@ sub tool
 }
 
 package GMB::TagEdit::EntryNumber;
-use Gtk2;
 use base 'Gtk2::SpinButton';
 
 sub new
@@ -1050,7 +1079,6 @@ sub output_nozero
 }
 
 package GMB::TagEdit::EntryBoolean;
-use Gtk2;
 use base 'Gtk2::CheckButton';
 
 sub new
@@ -1073,7 +1101,6 @@ sub get_text
 }
 
 package GMB::TagEdit::Combo;
-use Gtk2;
 use base 'Gtk2::Box';
 
 sub new
@@ -1116,8 +1143,7 @@ sub tool
 
 
 package GMB::TagEdit::EntryRating;
-use Gtk2;
-use base 'Gtk2::HBox';
+use base 'Gtk2::Box';
 
 sub new
 {	my ($class,$field,$IDs) = @_;
@@ -1173,7 +1199,6 @@ sub is_blank
 }
 
 package GMB::TagEdit::FlagList;
-use Gtk2;
 use base 'Gtk2::Box';
 
 sub new
@@ -1192,6 +1217,7 @@ sub new
 	$self->pack_start($entry,0,0,0);
 	$self->pack_start($add,0,0,0);
 	$add->signal_connect( button_press_event => sub { add_entry_text_cb($_[0]); $_[0]->grab_focus;1; } );
+	$add->signal_connect( clicked => \&add_entry_text_cb );
 	$button->signal_connect( clicked => \&popup_menu_cb);
 	$button->signal_connect( button_press_event => sub { popup_menu_cb($_[0]); $_[0]->grab_focus;1; } );
 	$entry->signal_connect( activate => \&add_entry_text_cb );
@@ -1219,10 +1245,7 @@ sub popup_add_menu
 {	my ($self,$widget)=@_;
 	my $cb= sub { $self->{selected}{ $_[1] }= 1; $self->update; };
 	my $menu=::MakeFlagMenu($self->{field},$cb);
-	$menu->show_all;
-	my $event=Gtk2->get_current_event;
-	my $button= $event->isa('Gtk2::Gdk::Event::Button') ? $event->button : 0;
-	$menu->popup(undef,undef,sub {::windowpos($_[0],$widget)},undef,$button,$event->time);
+	::PopupMenu($menu, posfunction=>sub {::windowpos($_[0],$widget)} );
 }
 
 sub popup_menu_cb
@@ -1238,10 +1261,7 @@ sub popup_menu_cb
 		$item->signal_connect(toggled => $cb,$key);
 		$menu->append($item);
 	}
-	$menu->show_all;
-	my $event=Gtk2->get_current_event;
-	my $button= $event->isa('Gtk2::Gdk::Event::Button') ? $event->button : 0;
-	$menu->popup(undef,undef,\&::menupos,undef,$button,$event->time);
+	::PopupMenu($menu);
 }
 
 sub update
@@ -1274,7 +1294,6 @@ sub set_text		# for setting from autofill-from-filename
 }
 
 package GMB::TagEdit::EntryMassList;	#for mass-editing fields with multiple values
-use Gtk2;
 use base 'Gtk2::Box';
 
 sub new
@@ -1286,6 +1305,7 @@ sub new
 	my $add= ::NewIconButton('gtk-add');
 	my $removeall= ::NewIconButton('gtk-clear', _"Remove all", \&clear);
 	$add->signal_connect( button_press_event => sub { add_entry_text_cb($_[0]); $_[0]->grab_focus;1; } );
+	$add->signal_connect( clicked => \&add_entry_text_cb );
 	for my $ref (['toadd',1,_"Add"],['toremove',-1,_"Remove"])
 	{	my ($key,$mode,$text)=@$ref;
 		my $label=$self->{$key}=Gtk2::Label->new;
@@ -1356,10 +1376,7 @@ sub popup_add_menu
 {	my ($self,$widget)=@_;
 	my $cb= sub { $self->{selected}{ $_[1] }= 1; $self->update; };
 	my $menu=::MakeFlagMenu($self->{field},$cb);
-	$menu->show_all;
-	my $event=Gtk2->get_current_event;
-	my $button= $event->isa('Gtk2::Gdk::Event::Button') ? $event->button : 0;
-	$menu->popup(undef,undef,sub {::windowpos($_[0],$widget)},undef,$button,$event->time);
+	::PopupMenu($menu, posfunction=>sub {::windowpos($_[0],$widget)} );
 }
 
 sub popup_menu_cb
@@ -1377,10 +1394,7 @@ sub popup_menu_cb
 		$item->signal_connect(toggled => $cb,$key);
 		$menu->append($item);
 	}
-	$menu->show_all;
-	my $event=Gtk2->get_current_event;
-	my $button= $event->isa('Gtk2::Gdk::Event::Button') ? $event->button : 0;
-	$menu->popup(undef,undef,\&::menupos,undef,$button,$event->time);
+	::PopupMenu($menu);
 	1;
 }
 
@@ -1407,11 +1421,10 @@ sub set_text		# for setting from autofill-from-filename
 }
 
 package EditTagSimple;
-use Gtk2;
+use base 'Gtk2::Box';
 
 use constant { TRUE  => 1, FALSE => 0, };
 
-use base 'Gtk2::VBox';
 sub new
 {	my ($class,$window,$ID) = @_;
 	my $self = bless Gtk2::VBox->new, $class;
@@ -1423,16 +1436,17 @@ sub new
 	$labelfile->set_selectable(TRUE);
 	$labelfile->set_line_wrap(TRUE);
 
+	my $sw=Gtk2::ScrolledWindow->new;
+	$sw->set_shadow_type('none');
+	$sw->set_policy('never', 'automatic');
+
 	my $table=Gtk2::Table->new (6, 2, FALSE);
+	$sw->add_with_viewport($table);
 	$self->{table}=$table;
 	$self->fill;
 
-	my $advanced=Gtk2::Button->new(_("Advanced Tag Editing").' ...');
-	$advanced->signal_connect( clicked => \&advanced_cb );
-
 	$self->pack_start($labelfile,FALSE,FALSE,1);
-	$self->pack_start($table, FALSE, TRUE, 2);
-	$self->pack_end($advanced, FALSE, FALSE, 2);
+	$self->pack_start($sw, TRUE, TRUE, 2);
 
 	return $self;
 }
@@ -1458,8 +1472,8 @@ sub fill
 	$table->show_all;
 }
 
-sub advanced_cb
-{	my $self=::find_ancestor($_[0],__PACKAGE__);
+sub advanced
+{	my $self=shift;
 	my $ID=$self->{ID};
 	my $dialog = Gtk2::Dialog->new (_"Advanced Tag Editing", $self->{window},
 		[qw/destroy-with-parent/],
@@ -1487,21 +1501,18 @@ sub advanced_cb
 sub save
 {	my $self=shift;
 	my $ID=$self->{ID};
-	my $errorsub=sub {::Retry_Dialog($_[0],$self->{window});};
 	my @modif;
 	while (my ($field,$entry)=each %{$self->{fields}})
 	{	push @modif,$field,$entry->get_text;
 	}
-	Songs::Set($ID,\@modif,error=>$errorsub) if @modif;
+	Songs::Set($ID,\@modif,window=>$self->{window}) if @modif;
 }
 
 
 ############################## Advanced tag editing ##############################
 
 package EditTag;
-use Gtk2;
-
-use base 'Gtk2::VBox';
+use base 'Gtk2::Box';
 
 sub new
 {	my ($class,$window,$ID) = @_;
@@ -1557,18 +1568,21 @@ sub save
 	for my $box (@{ $self->{boxes} })
 	{  $modified=1 if $box->save;
 	}
-	$self->{filetag}{errorsub}=sub {::Retry_Dialog($_[0],$self->{window});};
+	$self->{filetag}{errorsub}= sub
+	 {	my ($syserr,$details)= FileTag::Error_Message(@_);
+		return ::Retry_Dialog($syserr,_"Error writing tag", details=>$details, window=>$self->{window});
+	 };
 	$self->{filetag}->write_file if $modified && !$::CmdLine{ro} && !$::CmdLine{rotags};
 }
 
 package TagBox;
-use Gtk2;
+use base 'Gtk2::Box';
+
 use constant
 {	TRUE  => 1, FALSE => 0,
 	#contents of types hashes :
 	TAGNAME => 0, TAGORDER => 1, TAGTYPE => 2,
 };
-use base 'Gtk2::VBox';
 
 my %DataType;
 my %tagprop;
@@ -1881,9 +1895,9 @@ sub save
 }
 
 package TagBox_id3v1;
-use Gtk2;
+use base 'Gtk2::Box';
+
 use constant { TRUE  => 1, FALSE => 0 };
-use base 'Gtk2::VBox';
 
 sub new
 {	my ($class,$tag,$option)=@_;
@@ -1937,7 +1951,6 @@ sub save
 }
 
 package EntrySimple;
-use Gtk2;
 use base 'Gtk2::Entry';
 
 sub new
@@ -1958,7 +1971,6 @@ sub return_value
 }
 
 package EntryMultiLines;
-use Gtk2;
 use base 'Gtk2::ScrolledWindow';
 
 sub new
@@ -1990,7 +2002,6 @@ sub return_value
 }
 
 package EntryDouble;
-use Gtk2;
 use base 'Gtk2::Entry';
 
 sub new
@@ -2010,7 +2021,6 @@ sub return_value
 }
 
 package EntryNumber;
-use Gtk2;
 use base 'Gtk2::SpinButton';
 
 sub new
@@ -2029,7 +2039,6 @@ sub return_value
 }
 
 package EntryBoolean;
-use Gtk2;
 use base 'Gtk2::CheckButton';
 
 sub new
@@ -2046,7 +2055,6 @@ sub return_value
 	return $value;
 }
 package EntryCombo;
-use Gtk2;
 use base 'Gtk2::ComboBox';
 
 sub new
@@ -2075,7 +2083,7 @@ sub return_value
 }
 
 package EntryMulti;	#for id3v2 frames containing multiple fields
-use Gtk2;
+use base 'Gtk2::Frame';
 
 my %SUBTAGPROP; my $PICTYPE;
 INIT
@@ -2135,7 +2143,6 @@ INIT
 	);
 	$SUBTAGPROP{metadata_block_picture}=$SUBTAGPROP{APIC}; #for vorbis pictures
 }
-use base 'Gtk2::Frame';
 
 sub new
 {	my ($class,$values,$key,$name,$type,$realkey) = @_;
@@ -2180,7 +2187,6 @@ sub return_value
 }
 
 package EntryBinary;
-use Gtk2;
 use base 'Gtk2::Button';
 
 sub new
@@ -2226,8 +2232,7 @@ sub view
 }
 
 package EntryCover;
-use Gtk2;
-use base 'Gtk2::HBox';
+use base 'Gtk2::Box';
 
 sub new
 {	my $class = shift;
@@ -2294,10 +2299,10 @@ sub set
 	$img->parent->{pixdata}=$self->{value}; #for zoom on click
 }
 sub uri_dropped
-{	my $self=$_[0];
-	my ($file)=split /\x0d\x0a/,$_[2];
-	if ($file=~s#^file://##)
-	{	$self->load_file($file)
+{	my ($self,$type,$uri)=@_;
+	if ($uri=~s#^file://##)
+	{	my $file=::decode_url($uri);
+		$self->load_file($file);
 	}
 	#else #FIXME download http link
 }
@@ -2343,7 +2348,6 @@ sub _identify_pictype	#used only if $Gtk2::VERSION < 1.092
 }
 
 package EntryLyrics;
-use Gtk2;
 use base 'Gtk2::Button';
 
 sub new
